diff -u -b /e/source/winehq/wine/dlls/shell32/shell32.spec shell32/shell32.spec
--- /e/source/winehq/wine/dlls/shell32/shell32.spec	Thu Jan  8 21:32:16 2004
+++ shell32/shell32.spec	Wed Jan 14 12:04:36 2004
@@ -52,7 +52,7 @@
   56 stdcall PathUnquoteSpaces(str) PathUnquoteSpacesAW
   57 stdcall PathGetDriveNumber (str) PathGetDriveNumberAW
   58 stdcall ParseField(str long ptr long) ParseFieldAW
-  59 stub RestartDialog@12
+  59 stdcall RestartDialog(long long long)
   60 stdcall ExitWindowsDialog(long)
   61 stdcall RunFileDlg(long long long str str long)
   62 stdcall PickIconDlg(long long long long)
@@ -309,6 +309,8 @@
 
 # >= NT5
  714 stdcall @(ptr)SHELL32_714 # PathIsTemporaryW
+
+ 730 stdcall RestartDialogEx(long long long long)
 
 1217 stub FOOBAR1217   # no joke! This is the real name!!
 

diff -u -b /e/source/winehq/wine/dlls/shell32/dialogs.c shell32/dialogs.c
--- /e/source/winehq/wine/dlls/shell32/dialogs.c	Wed Jan 14 12:03:20 2004
+++ shell32/dialogs.c	Thu Jan  8 21:27:36 2004
@@ -372,6 +372,63 @@
     free (pszList) ;
     }
 
+
+/*************************************************************************
+ * RestartWindowsDialog				[SHELL32.730]
+ */
+
+int WINAPI RestartDialogEx(HWND hwndOwner, LPCWSTR lpwstrReason, UINT uFlags, UINT uReason)
+{
+    TRACE("(%p)\n", hwndOwner);
+
+    /*FIXME: use uReason */
+
+    if (MessageBoxA(hwndOwner, "Do you want to restart the system?", "Restart", MB_YESNO|MB_ICONQUESTION) == IDYES)
+    {
+	if (SHELL_OsIsUnicode())
+	{
+	    HANDLE hToken;
+	    TOKEN_PRIVILEGES npr = {1, {{{0, 0}, SE_PRIVILEGE_ENABLED}}};
+
+	    /* enable shutdown privilege for current process */
+	    OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &hToken);
+	    LookupPrivilegeValueA(0, "SeShutdownPrivilege", &npr.Privileges[0].Luid);
+	    AdjustTokenPrivileges(hToken, FALSE, &npr, 0, 0, 0);
+	    CloseHandle(hToken);
+	}
+
+	ExitWindowsEx(EWX_REBOOT, 0);
+    }
+
+    return 0;
+}
+
+
+/*************************************************************************
+ * RestartWindowsDialog				[SHELL32.59]
+ */
+
+int WINAPI RestartDialog(HWND hwndOwner, LPCSTR lpstrReason, UINT uFlags)
+{
+    LPWSTR lpwcsReason;
+    int len, ret;
+
+    if (lpstrReason) {
+	len = MultiByteToWideChar(CP_ACP, 0, lpstrReason, -1, NULL, 0);
+	lpwcsReason = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+	MultiByteToWideChar(CP_ACP, 0, lpstrReason, -1, lpwcsReason, len);
+    } else
+	lpwcsReason = NULL;
+
+    ret = RestartDialogEx(hwndOwner, lpwcsReason, uFlags, 0);
+
+    if (lpwcsReason)
+	HeapFree(GetProcessHeap(), 0, lpwcsReason);
+
+    return ret;
+}
+
+
 /*************************************************************************
  * ExitWindowsDialog				[SHELL32.60]
  *
@@ -381,8 +438,23 @@
 void WINAPI ExitWindowsDialog (HWND hWndOwner)
 {
 	TRACE("(%p)\n", hWndOwner);
-	if (MessageBoxA( hWndOwner, "Do you want to exit WINE?", "Shutdown", MB_YESNO|MB_ICONQUESTION) == IDYES)
+
+    if (MessageBoxA(hWndOwner, "Do you want to shutdown?", "Shutdown", MB_YESNO|MB_ICONQUESTION) == IDYES)
+    {
+	if (SHELL_OsIsUnicode())
 	{
-	  SendMessageA ( hWndOwner, WM_QUIT, 0, 0);
+	    HANDLE hToken;
+	    TOKEN_PRIVILEGES npr = {1, {{{0, 0}, SE_PRIVILEGE_ENABLED}}};
+
+	    /* enable shutdown privilege for current process */
+	    OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &hToken);
+	    LookupPrivilegeValueA(0, "SeShutdownPrivilege", &npr.Privileges[0].Luid);
+	    AdjustTokenPrivileges(hToken, FALSE, &npr, 0, 0, 0);
+	    CloseHandle(hToken);
+
+	    ExitWindowsEx(EWX_SHUTDOWN, 0);
+	}
+	else
+	    SendMessageA(hWndOwner, WM_QUIT, 0, 0);
 	}
 }
